/* 
 * The contents of this file are subject to the GrammarSoft Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.grammarsoft.com/GSPL
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the VISL CG Parser.
 * 
 * The Initial Developer of the Original Code is Martin Carlsen, 
 * GrammarSoft, and SDU Odense University.  
 * Portions created by Martin Carlsen, Grammarsoft, and SDU Odense 
 * University are Copyright (C) 1999-2000 Martin Carlsen, GrammarSoft, 
 * and SDU Odense University.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the GSPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the GSPL or the
 * GPL.
 */
#include "Options.hh"
#include <map>
#include <string>

string Options::describe() const 
{
  string s = "";
  for(map<string,string>::iterator 
	i = valid_options->begin(),
	z = valid_options->end();
      i!=z; ++i)
    {
      s += (*i).second + "\n";
    }
  return s;
}

bool Options::getFlag(string name, bool def=false) const
{
  if( flags->find(name) == flags->end() ) return def;
  return (*flags)[name];
}
string Options::getSetting(string name, string def="") const
{
  if( settings->find(name) == settings->end() ) return def;
  return (*settings)[name];
}

void Options::parse(int argc, char * argv[])
{
  flags = new map<string,bool>();
  settings = new map<string,string>();
  params = new list<string>();
  for(int i=1; i<argc; ++i)
    {
      bool flagvalue = true;
      
      switch(argv[i][0])
	{
	case 0:
	  cerr << "Opions::Options(...) We should never be here.\n";
	  exit(0);
	case '+': // e.g. +x
	  flagvalue = false;
	case '-':
	  if(argv[i][1] != '-') // e.g. -V
	    {
	      string * s = new string(argv[i]+1);
	      string name = *s;
	      (*flags)[name] = flagvalue;
	      //cout << name << " = " << (flagvalue?"true":"false") << "\n";
	      break;
	    }
	  else
	    {
	      string * s = new string(argv[i]+2);
	      string name = *s;
	      string::size_type p = name.find('=');
	      if(p == string::npos)
		{
		  if(i+1<argc && argv[i+1][0]!='-' && argv[i+1][0]!='+') // e.g. --grammar dancg
		    {
		      //cout << name << " = \n";
		      ++i;
		      string * v = new string(argv[i]);
		      string value = *v;
		      (*settings)[name] = value;
		    }
		  else // e.g. --license
		    {
		      (*settings)[name] = "";
		    }
		}
	      else // e.g. --grammar=dancg
		{
		  (*settings)[name.substr(0,p)] = name.substr(p+1);
		  //cout << name.substr(0,p) << " = " << name.substr(p+1) << "\n";
		}
	    }
	  break;
	default: // e.g. dancg
	  {
	    string * s = new string(argv[i]);
	    string name = *s;
	    //cout << name << "\n";
	    params->push_back(name);
	  }
	  break;
	}
    }
}
