/* 
 * The contents of this file are subject to the GrammarSoft Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.grammarsoft.com/GSPL
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the VISL CG Parser.
 * 
 * The Initial Developer of the Original Code is Martin Carlsen, 
 * GrammarSoft, and SDU Odense University.  
 * Portions created by Martin Carlsen, Grammarsoft, and SDU Odense 
 * University are Copyright (C) 1999-2000 Martin Carlsen, GrammarSoft, 
 * and SDU Odense University.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the GSPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the GSPL or the
 * GPL.
 */
#include <vector>
#include <list>
#include <string>

#include "cg.data_structures.hh"

#include "cohorts.hh"

#include "CGCompiler.hh"

extern TagRegister * tag_register;

#define IERR(msg) {\
  cerr << (msg) << "\n";\
  return cohorts;\
}

string * startTag = new string(">>>");
string * endTag = new string("<<<");

/**
 * (2.3.2: Fixed tags. Auxiliary ending markers.)
 */
void addEndingMarker(Cohort* cohort)
{
  if(cohort->getReadings()->size()==0)
    {
      cohort->getReadings()->push_back(new Combined());
    }
  for(list<Combined*>::iterator
	i = cohort->getReadings()->begin(),
	z = cohort->getReadings()->end();
      i!=z; ++i)
    {
      (*i)->push_back(tag_register->add(*endTag));
    }
}

vector<Cohort*>* readCohorts(taglist<int>* delims)
{
  
  vector<Cohort*>* cohorts = new vector<Cohort*>();
  
  // (2.3.2: Fixed tags. Auxiliary beginning marker.)
  Combined * auxBeginTag = new Combined();
  auxBeginTag->push_back(tag_register->add(*startTag));
  list<Combined*>* auxBeginReadings = new list<Combined*>();
  auxBeginReadings->push_back(auxBeginTag);
  Cohort * auxBegin = new Cohort(">>>", auxBeginReadings);
  cohorts->push_back(auxBegin);
  
  Cohort * cohort = 0;
  string tmp = "";
  char c = 0;
  cin.get(c);
  
  while(true)
    {
      if(cohort)
	{
	  cohorts->push_back(cohort);
	  
	  // is the cohort a delimiter?
	  for(taglist<int>::iterator 
		i = delims->begin(),
		z = delims->end();
	      i!=z; ++i)
	    {
	      int wf = tag_register->get(cohort->getWordform());
	      int delim = *i;
	      if(wf == delim)
		{
		  if(c!=0) cin.putback(c);
		  addEndingMarker(cohort);
		  return cohorts;
		}
	    }
	}
      if(c==0) break;
      
      // read wordform: "<wordform>"
      if(c == '\"')
	{
	  tmp = c;
	  while(cin.get(c) && c!='\n' && c!=0) tmp += c;
	  //cout << "WORDFORM: " << tmp << "\n";
	  cohort = new Cohort(tmp, new list<Combined*>());
	}
      else if(c=='\n' || c==0) 
	{
	  // blank line or end of input
	  cin.get(c);
	  //cout << c << "\n";
	  continue;
	}
      
      // read first character of first reading.
      if(!cin.get(c)) c = 0;
      
      // read readings
      // reading = (\S+\W+)+
      // readings = ({reading}\S*\n)*
      while(c!='\"' && c!=0)
	{
	  // read one reading
	  Combined * reading = new Combined();
	  while(c==' ' || c=='\t')
	    {
	      // read blanks
	      while(cin.get(c) && (c==' ' || c=='\t'));
	      tmp = c;
	      if(c=='\n') break; // end of line. read next reading or cohort.
	      
	      // read tag
	      while(cin.get(c) && c!=' ' && c!='\t' && c!='\n' && c!=0) if(c!='\\') tmp += c;
	      
	      // add tag to reading.
	      reading->push_back(tag_register->add(tmp));
	      
	      if(c=='\n') break; // end of line. read next reading or cohort.
	    }
	  // add reading to cohort
	  if(reading->size() > 0) cohort->getReadings()->push_back(reading);
	  else delete reading;
	  //cout << "LINE: " << reading->size() << " tags on line\n";
	  if(!cin.get(c)){
	    c = 0;
	    break;
	  }
	}
    }
  if(!cohort) return 0;
  CGCompiler::expandCohorts(cohorts);
  addEndingMarker(cohort);
  return cohorts;
}
